//  Created by D. Byrnes.

/** Header file for an array-based implementation of the ADT sack. */
#include <iostream>
#include <string>

using namespace std;

#ifndef ARRAY_Sack_
#define ARRAY_Sack_

#include "SackInterface.h"

template<class T>
class ArraySack : public SackInterface<T>
{
public:
	ArraySack();
	int getCurrentSize() const;
	bool isEmpty() const;
	bool add(const T& newEntry);
	bool remove(const T& anEntry);
	void clear();
	bool contains(const T& anEntry) const;
	
private:
    static const int DEFAULT_CAPACITY = 6; // Small size to test for a full Sack
    T items[DEFAULT_CAPACITY];      // Array of Sack items
    int itemCount;                         // Current count of Sack items
    int maxItems;                          // Max capacity of the Sack
}; // end ArraySack

#endif

template<class T>
ArraySack<T>::ArraySack() : itemCount(0), maxItems(DEFAULT_CAPACITY)
{
}  // end default constructor

template<class T>
int ArraySack<T>::getCurrentSize() const
{
    return itemCount;
}  // end getCurrentSize

template<class T>
bool ArraySack<T>::isEmpty() const
{
    return itemCount == 0;
}  // end isEmpty

template<class T>
bool ArraySack<T>::add(const T& newEntry)
{
    bool hasRoomToAdd = (itemCount < maxItems);
    if (hasRoomToAdd)
    {
        items[itemCount] = newEntry;
        itemCount++;
    }  // end if
    
    return hasRoomToAdd;
}  // end add

template<class T>
bool ArraySack<T>::remove(const T& anEntry)
{
    int locatedIndex;
    
    for (int i = 0; i < itemCount; i++)
    {
        if (anEntry == items[i])
        {
            locatedIndex = i;;
        }
    }
    
    bool canRemoveItem = !isEmpty() && (locatedIndex > -1);
    if (canRemoveItem)
    {
        itemCount--;
        items[locatedIndex] = items[itemCount];
    }  // end if
    
    return canRemoveItem;
}  // end remove

template<class T>
void ArraySack<T>::clear()
{
    itemCount = 0;
}  // end clear

 template<class T>
 bool ArraySack<T>::contains(const T& anEntry) const
 {
 bool found = false;
 for (int i = 0; !found && (i < itemCount); i++)
 {
     if (anEntry == items[i])
     {
         found = true;
     } // end if
 } // end for
 
 return found;
 }  // end contains


