//  Idea from Frank M. Carrano and Timothy M. Henry.
//  Copyright (c) 2017 Pearson Education, Hoboken, New Jersey.

/** Modified by D. Byrnes for a simplified Sack class
    A Sack holds a bunch of items of the same type
 */
#ifndef Sack_INTERFACE_
#define Sack_INTERFACE_

#include <vector>

template<class T>
class SackInterface
{
public:
   /** Gets the current number of entries in this Sack.
    @pre: none
    @post: returns the number of items in the Sack object
    @return The integer number of entries currently in the Sack. */
   virtual int getCurrentSize() const = 0;
   
   /** Sees whether this Sack is empty.
    @pre: none
    @post: returns true iff Sack object is empty
    @return True if the Sack is empty, or false if not. */
   virtual bool isEmpty() const = 0;
   
   /** Adds a new entry to this Sack.
    @pre:none
    @post:  If successful, newEntry is stored in the Sack and
       the count of items in the Sack has increased by 1.
    @param newEntry  The object to be added as a new entry.
    @return  True if addition was successful, or false if not. */
   virtual bool add(const T& newEntry) = 0;
   
   /** Removes one occurrence of a given entry from this Sack,
       if possible.
    @PRE: NONE
    @post  If successful, anEntry has been removed from the Sack
       and the count of items in the Sack has decreased by 1.
    @param anEntry  The entry to be removed.
    @return  True if removal was successful, or false if not. */
   virtual bool remove(const T& anEntry) = 0;
   
   /** Removes all entries from this Sack.
    @pre: none
    @post  Sack contains no items, and the count of items is 0. */
   virtual void clear() = 0;
   
   /** Tests whether this Sack contains a given entry.
    @pre: none
    @post: returns true iff anEntry is in the Sack object
    @param anEntry  The entry to locate.
    @return  True if Sack contains anEntry, or false otherwise. */
   virtual bool contains(const T& anEntry) const = 0;
   
   /** Destroys object and frees memory allocated by object.
    @pre: none
    @post: Sack object is destroyed
 */
   virtual ~SackInterface () { }
   
}; // end SackInterface
#endif